package com.kim.apidoc.core;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.kim.apidoc.core.common.helper.FileHelper;
import com.kim.apidoc.core.render.AsciiDocRender;
import com.kim.apidoc.core.render.MarkdownRender;
import com.kim.apidoc.core.render.PostmanRender;
import com.kim.apidoc.core.render.ProjectRender;
import lombok.Getter;
import lombok.Setter;

import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.List;
import java.util.Map;

import static com.kim.apidoc.core.common.Constants.MARKDOWN_TEMPLATE;

/**
 * The type Context.
 *
 * @author fengyuchenglun
 * @version 1.0.0
 */
@Getter
public class Context {

    /**
     * 默认-节点索引
     */
    public static final Integer DEFAULT_NODE_INDEX = 99;
    /**
     * 默认-项目编号
     */
    public static final String DEFAULT_PROJECT_ID = "api";
    /**
     * 默认-编译路径
     */
    public static final String DEFAULT_BUILD_PATH = "build";
    /**
     * 默认-代码结构
     */
    public static final String DEFAULT_CODE_STRUCTURE = "src/main/java";

    /**
     * 设置当前解析框架
     */
    @Setter
    private String framework;

    /**
     * The Renders.
     */
    @Setter
    private List<ProjectRender> renders = Lists.newArrayList(
            new AsciiDocRender(),
            new PostmanRender(),
            new MarkdownRender());

    /**
     * 编译目录
     */
    @Setter
    private Path buildPath = Paths.get(DEFAULT_BUILD_PATH);

    /**
     * 源码目录
     */
    private List<Path> sources = Lists.newArrayList();

    /**
     * 依赖源码
     */
    private List<Path> dependencies = Lists.newArrayList();

    /**
     * 依赖jar包
     */
    private List<Path> jars = Lists.newArrayList();

    /**
     * 项目编号
     */
    @Setter
    private String id = DEFAULT_PROJECT_ID;
    /**
     * 名称
     */
    @Setter
    private String name;
    /**
     * 描述
     */
    @Setter
    private String description;
    /**
     * 版本
     */
    @Setter
    private String version;

    /**
     * 渲染html时的css
     */
    @Setter
    private String css;


    /**
     * markdown模版文件路径
     */
    @Setter
    private String markdownTemplate = MARKDOWN_TEMPLATE;

    /**
     * 自定义扩展参数
     */
    private Map<String, Object> ext = Maps.newHashMap();

    /**
     * Add source.
     *
     * @param path the path
     */
    public void addSource(Path path) {
        sources.add(path);
        sources.addAll(FileHelper.find(path, DEFAULT_CODE_STRUCTURE));
        addDependency(path);
    }

    /**
     * Add dependency.
     *
     * @param path the path
     */
    public void addDependency(Path path) {
        dependencies.add(path);
        dependencies.addAll(FileHelper.find(path, DEFAULT_CODE_STRUCTURE));
    }

    /**
     * Add jar.
     *
     * @param path the path
     */
    public void addJar(Path path) {
        jars.add(path);
    }


}
