package com.kim.apidoc.core.parser;

import com.kim.apidoc.core.schema.Chapter;
import com.kim.apidoc.core.schema.Node;
import com.kim.apidoc.core.schema.Project;
import com.kim.apidoc.core.schema.Section;
import com.kim.apidoc.core.common.helper.OptionalHelper;
import com.github.javaparser.ast.body.ClassOrInterfaceDeclaration;
import com.github.javaparser.ast.body.MethodDeclaration;
import com.github.javaparser.ast.visitor.VoidVisitorAdapter;

/**
 * The type Visitor parser.
 */
public class VisitorParser extends VoidVisitorAdapter<Node> {

    /**
     * The Parser strategy.
     */
    private ParserStrategy parserStrategy;

    /**
     * Sets parser strategy.
     *
     * @param parserStrategy the parser strategy
     */
    public void setParserStrategy(ParserStrategy parserStrategy) {
        this.parserStrategy = parserStrategy;
    }

    /**
     * 类或者接口声明
     * @param classOrInterfaceDeclaration
     * @param arg
     */
    @Override
    public void visit(final ClassOrInterfaceDeclaration classOrInterfaceDeclaration, final Node arg) {

        if (arg instanceof Project && parserStrategy.accept(classOrInterfaceDeclaration)) {
            Project project = (Project) arg;
            // 章节
            Chapter chapter = new Chapter();
            classOrInterfaceDeclaration.getFullyQualifiedName().ifPresent(chapter::setId);
            chapter.setName(classOrInterfaceDeclaration.getNameAsString());
            classOrInterfaceDeclaration.getComment().ifPresent(chapter::accept);

            OptionalHelper.any(chapter.getTag("book"),chapter.getTag("group"))
                    .ifPresent(tag -> chapter.setBookName(tag.getContent()));

            parserStrategy.visit(classOrInterfaceDeclaration, chapter);
            project.addChapter(chapter);
            super.visit(classOrInterfaceDeclaration, chapter);
        }
    }

    /**
     * 方法声明
     * @param methodDeclaration
     * @param arg
     */
    @Override
    public void visit(final MethodDeclaration methodDeclaration, final Node arg) {
        if (arg instanceof Chapter && parserStrategy.accept(methodDeclaration)) {
            Chapter chapter = (Chapter) arg;
            Section section = new Section();
            section.setId(methodDeclaration.getNameAsString());
            section.setName(methodDeclaration.getNameAsString());
            section.setIndex(chapter.getSections().size());
            methodDeclaration.getComment().ifPresent(section::accept);

            parserStrategy.visit(methodDeclaration, chapter, section);
            chapter.getSections().add(section);
            super.visit(methodDeclaration, section);
        }
    }

}
